package java.util;

import java.lang.invoke.SerializedLambda;
import java.util.function.Consumer;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.io.IOException;
import java.io.Serializable;

public class TreeMap<K, V> extends AbstractMap<K, V> implements NavigableMap<K, V>, Cloneable, Serializable
{
    private final Comparator<? super K> comparator;
    private transient TreeMap.Entry<K, V> root;
    private transient int size;
    private transient int modCount;
    private transient TreeMap.EntrySet entrySet;
    private transient TreeMap.KeySet<K> navigableKeySet;
    private transient NavigableMap<K, V> descendingMap;
    private static final Object UNBOUNDED;
    private static final boolean RED = false;
    private static final boolean BLACK = true;
    private static final long serialVersionUID = 919286545866124006L;
    
    public TreeMap() {
        this.size = 0;
        this.modCount = 0;
        this.comparator = null;
    }
    
    public TreeMap(Comparator<? super K> comparator) {
        this.size = 0;
        this.modCount = 0;
        this.comparator = comparator;
    }
    
    public TreeMap(Map<? extends K, ? extends V> map) {
        this.size = 0;
        this.modCount = 0;
        this.comparator = null;
        this.putAll(map);
    }
    
    public TreeMap(SortedMap<K, ? extends V> sortedMap) {
        this.size = 0;
        this.modCount = 0;
        this.comparator = sortedMap.comparator();
        try {
            this.buildFromSorted(sortedMap.size(), sortedMap.entrySet().iterator(), null, null);
        }
        catch (IOException ex) {}
        catch (ClassNotFoundException ex2) {}
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean containsKey(Object o) {
        return this.getEntry(o) != null;
    }
    
    @Override
    public boolean containsValue(Object o) {
        TreeMap.Entry<Object, Object> entry = this.getFirstEntry();
        while (entry != null) {
            if (valEquals(o, entry.value)) {
                return true;
            }
            entry = successor(entry);
        }
        return false;
    }
    
    @Override
    public V get(Object o) {
        TreeMap.Entry<K, V> entry = this.getEntry(o);
        return (entry == null) ? null : entry.value;
    }
    
    @Override
    public Comparator<? super K> comparator() {
        return this.comparator;
    }
    
    @Override
    public K firstKey() {
        return key(this.getFirstEntry());
    }
    
    @Override
    public K lastKey() {
        return key(this.getLastEntry());
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        int size = map.size();
        if (this.size == 0 && size != 0 && map instanceof SortedMap) {
            Comparator<Object> comparator = ((SortedMap)map).comparator();
            if (comparator == this.comparator || (comparator != null && comparator.equals(this.comparator))) {
                ++this.modCount;
                try {
                    this.buildFromSorted(size, map.entrySet().iterator(), null, null);
                }
                catch (IOException ex) {}
                catch (ClassNotFoundException ex2) {}
                return;
            }
        }
        super.putAll(map);
    }
    
    final TreeMap.Entry<K, V> getEntry(Object o) {
        if (this.comparator != null) {
            return this.getEntryUsingComparator(o);
        }
        if (o == null) {
            throw new NullPointerException();
        }
        Comparable<TreeMap.Entry> comparable = (Comparable)o;
        TreeMap.Entry<Object, Object> entry = this.root;
        while (entry != null) {
            int compareTo = comparable.compareTo(entry.key);
            if (compareTo < 0) {
                entry = entry.left;
            }
            else if (compareTo > 0) {
                entry = entry.right;
            }
            else {
                return entry;
            }
        }
        return null;
    }
    
    final TreeMap.Entry<K, V> getEntryUsingComparator(Object o) {
        Comparator comparator = this.comparator;
        if (comparator != null) {
            TreeMap.Entry<Object, Object> entry = this.root;
            while (entry != null) {
                int compare = comparator.compare(o, entry.key);
                if (compare < 0) {
                    entry = entry.left;
                }
                else if (compare > 0) {
                    entry = entry.right;
                }
                else {
                    return entry;
                }
            }
        }
        return null;
    }
    
    final TreeMap.Entry<K, V> getCeilingEntry(K k) {
        TreeMap.Entry<Object, Object> entry = this.root;
        while (entry != null) {
            int compare = this.compare(k, entry.key);
            if (compare < 0) {
                if (entry.left != null) {
                    entry = entry.left;
                }
                else {
                    return entry;
                }
            }
            else if (compare > 0) {
                if (entry.right != null) {
                    entry = entry.right;
                }
                else {
                    TreeMap.Entry<Object, Object> entry2;
                    for (entry2 = entry.parent, ; entry2 != null && entry == entry2.right; entry = entry2, entry2 = entry2.parent) {}
                    return entry2;
                }
            }
            else {
                return entry;
            }
        }
        return null;
    }
    
    final TreeMap.Entry<K, V> getFloorEntry(K k) {
        TreeMap.Entry<Object, Object> entry = this.root;
        while (entry != null) {
            int compare = this.compare(k, entry.key);
            if (compare > 0) {
                if (entry.right != null) {
                    entry = entry.right;
                }
                else {
                    return entry;
                }
            }
            else if (compare < 0) {
                if (entry.left != null) {
                    entry = entry.left;
                }
                else {
                    TreeMap.Entry<Object, Object> entry2;
                    for (entry2 = entry.parent, ; entry2 != null && entry == entry2.left; entry = entry2, entry2 = entry2.parent) {}
                    return entry2;
                }
            }
            else {
                return entry;
            }
        }
        return null;
    }
    
    final TreeMap.Entry<K, V> getHigherEntry(K k) {
        TreeMap.Entry<Object, Object> entry = this.root;
        while (entry != null) {
            if (this.compare(k, entry.key) < 0) {
                if (entry.left != null) {
                    entry = entry.left;
                }
                else {
                    return entry;
                }
            }
            else if (entry.right != null) {
                entry = entry.right;
            }
            else {
                TreeMap.Entry<Object, Object> entry2;
                for (entry2 = entry.parent, ; entry2 != null && entry == entry2.right; entry = entry2, entry2 = entry2.parent) {}
                return entry2;
            }
        }
        return null;
    }
    
    final TreeMap.Entry<K, V> getLowerEntry(K k) {
        TreeMap.Entry<Object, Object> entry = this.root;
        while (entry != null) {
            if (this.compare(k, entry.key) > 0) {
                if (entry.right != null) {
                    entry = entry.right;
                }
                else {
                    return entry;
                }
            }
            else if (entry.left != null) {
                entry = entry.left;
            }
            else {
                TreeMap.Entry<Object, Object> entry2;
                for (entry2 = entry.parent, ; entry2 != null && entry == entry2.left; entry = entry2, entry2 = entry2.parent) {}
                return entry2;
            }
        }
        return null;
    }
    
    @Override
    public V put(K k, V v) {
        TreeMap.Entry<V, Object> entry = this.root;
        if (entry == null) {
            this.compare(k, k);
            this.root = new TreeMap.Entry(k, v);
            this.size = 1;
            ++this.modCount;
            return null;
        }
        Comparator comparator = this.comparator;
        int n;
        if (comparator != null) {
            do {
                n = comparator.compare(k, entry.key);
                if (n < 0) {
                    entry = entry.left;
                }
                else if (n > 0) {
                    entry = entry.right;
                }
                else {
                    return entry.setValue(v);
                }
            } while (entry != null);
        }
        else {
            if (k == null) {
                throw new NullPointerException();
            }
            Comparable<TreeMap.Entry> comparable = (Comparable)k;
            do {
                n = comparable.compareTo(entry.key);
                if (n < 0) {
                    entry = entry.left;
                }
                else if (n > 0) {
                    entry = entry.right;
                }
                else {
                    return entry.setValue(v);
                }
            } while (entry != null);
        }
        TreeMap.Entry o = new TreeMap.Entry<Object, Object>(k, v, entry);
        if (n < 0) {
            entry.left = o;
        }
        else {
            entry.right = o;
        }
        this.fixAfterInsertion(o);
        ++this.size;
        ++this.modCount;
        return null;
    }
    
    @Override
    public V remove(Object o) {
        TreeMap.Entry<K, V> entry = this.getEntry(o);
        if (entry == null) {
            return null;
        }
        Object value = entry.value;
        this.deleteEntry(entry);
        return value;
    }
    
    @Override
    public void clear() {
        ++this.modCount;
        this.size = 0;
        this.root = null;
    }
    
    public Object clone() {
        TreeMap<Object, Object> treeMap;
        try {
            treeMap = (TreeMap)super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
        treeMap.root = null;
        treeMap.size = 0;
        treeMap.modCount = 0;
        treeMap.entrySet = null;
        treeMap.navigableKeySet = null;
        treeMap.descendingMap = null;
        try {
            treeMap.buildFromSorted(this.size, this.entrySet().iterator(), null, null);
        }
        catch (IOException ex) {}
        catch (ClassNotFoundException ex2) {}
        return treeMap;
    }
    
    @Override
    public Map.Entry<K, V> firstEntry() {
        return exportEntry(this.getFirstEntry());
    }
    
    @Override
    public Map.Entry<K, V> lastEntry() {
        return exportEntry(this.getLastEntry());
    }
    
    @Override
    public Map.Entry<K, V> pollFirstEntry() {
        TreeMap.Entry<K, V> firstEntry = this.getFirstEntry();
        Map.Entry<K, V> exportEntry = exportEntry(firstEntry);
        if (firstEntry != null) {
            this.deleteEntry(firstEntry);
            return exportEntry;
        }
        return exportEntry;
    }
    
    @Override
    public Map.Entry<K, V> pollLastEntry() {
        TreeMap.Entry<K, V> lastEntry = this.getLastEntry();
        Map.Entry<K, V> exportEntry = exportEntry(lastEntry);
        if (lastEntry != null) {
            this.deleteEntry(lastEntry);
            return exportEntry;
        }
        return exportEntry;
    }
    
    @Override
    public Map.Entry<K, V> lowerEntry(K k) {
        return exportEntry(this.getLowerEntry(k));
    }
    
    @Override
    public K lowerKey(K k) {
        return keyOrNull(this.getLowerEntry(k));
    }
    
    @Override
    public Map.Entry<K, V> floorEntry(K k) {
        return exportEntry(this.getFloorEntry(k));
    }
    
    @Override
    public K floorKey(K k) {
        return keyOrNull(this.getFloorEntry(k));
    }
    
    @Override
    public Map.Entry<K, V> ceilingEntry(K k) {
        return exportEntry(this.getCeilingEntry(k));
    }
    
    @Override
    public K ceilingKey(K k) {
        return keyOrNull(this.getCeilingEntry(k));
    }
    
    @Override
    public Map.Entry<K, V> higherEntry(K k) {
        return exportEntry(this.getHigherEntry(k));
    }
    
    @Override
    public K higherKey(K k) {
        return keyOrNull(this.getHigherEntry(k));
    }
    
    @Override
    public Set<K> keySet() {
        return this.navigableKeySet();
    }
    
    @Override
    public NavigableSet<K> navigableKeySet() {
        TreeMap.KeySet<Object> navigableKeySet = this.navigableKeySet;
        return (navigableKeySet != null) ? navigableKeySet : (this.navigableKeySet = new TreeMap.KeySet(this));
    }
    
    @Override
    public NavigableSet<K> descendingKeySet() {
        return this.descendingMap().navigableKeySet();
    }
    
    @Override
    public Collection<V> values() {
        Collection<Object> values = this.values;
        if (values == null) {
            values = new TreeMap.Values();
            this.values = values;
            return values;
        }
        return values;
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        TreeMap.EntrySet entrySet = this.entrySet;
        return (entrySet != null) ? entrySet : (this.entrySet = new TreeMap.EntrySet());
    }
    
    @Override
    public NavigableMap<K, V> descendingMap() {
        NavigableMap<Object, Object> descendingMap = this.descendingMap;
        return (descendingMap != null) ? descendingMap : (this.descendingMap = new TreeMap.DescendingSubMap(this, true, null, true, true, null, true));
    }
    
    @Override
    public NavigableMap<K, V> subMap(K k, boolean b, K i, boolean b2) {
        return new TreeMap.AscendingSubMap(this, false, k, b, false, i, b2);
    }
    
    @Override
    public NavigableMap<K, V> headMap(K k, boolean b) {
        return new TreeMap.AscendingSubMap(this, true, null, true, false, k, b);
    }
    
    @Override
    public NavigableMap<K, V> tailMap(K k, boolean b) {
        return new TreeMap.AscendingSubMap(this, false, k, b, true, null, true);
    }
    
    @Override
    public SortedMap<K, V> subMap(K k, K i) {
        return this.subMap(k, true, i, false);
    }
    
    @Override
    public SortedMap<K, V> headMap(K k) {
        return this.headMap(k, false);
    }
    
    @Override
    public SortedMap<K, V> tailMap(K k) {
        return this.tailMap(k, true);
    }
    
    @Override
    public boolean replace(K k, V v, V value) {
        TreeMap.Entry<K, V> entry = this.getEntry(k);
        if (entry != null && Objects.equals(v, entry.value)) {
            entry.value = value;
            return true;
        }
        return false;
    }
    
    @Override
    public V replace(K k, V value) {
        TreeMap.Entry<K, V> entry = this.getEntry(k);
        if (entry != null) {
            Object value2 = entry.value;
            entry.value = value;
            return value2;
        }
        return null;
    }
    
    @Override
    public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        Objects.requireNonNull(biConsumer);
        int modCount = this.modCount;
        TreeMap.Entry<Object, Object> entry = this.getFirstEntry();
        while (entry != null) {
            biConsumer.accept(entry.key, entry.value);
            if (modCount != this.modCount) {
                throw new ConcurrentModificationException();
            }
            entry = successor(entry);
        }
    }
    
    @Override
    public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        int modCount = this.modCount;
        TreeMap.Entry<Object, Object> entry = this.getFirstEntry();
        while (entry != null) {
            entry.value = biFunction.apply(entry.key, entry.value);
            if (modCount != this.modCount) {
                throw new ConcurrentModificationException();
            }
            entry = successor(entry);
        }
    }
    
    Iterator<K> keyIterator() {
        return new TreeMap.KeyIterator(this.getFirstEntry());
    }
    
    Iterator<K> descendingKeyIterator() {
        return new TreeMap.DescendingKeyIterator(this.getLastEntry());
    }
    
    final int compare(Object o, Object o2) {
        return (this.comparator == null) ? ((Comparable)o).compareTo(o2) : this.comparator.compare(o, o2);
    }
    
    static final boolean valEquals(Object o, Object obj) {
        return (o == null) ? (obj == null) : o.equals(obj);
    }
    
    static <K, V> Map.Entry<K, V> exportEntry(TreeMap.Entry<K, V> entry) {
        return (entry == null) ? null : new AbstractMap.SimpleImmutableEntry(entry);
    }
    
    static <K, V> K keyOrNull(TreeMap.Entry<K, V> entry) {
        return (entry == null) ? null : entry.key;
    }
    
    static <K> K key(TreeMap.Entry<K, ?> entry) {
        if (entry == null) {
            throw new NoSuchElementException();
        }
        return entry.key;
    }
    
    final TreeMap.Entry<K, V> getFirstEntry() {
        TreeMap.Entry<Object, Object> entry = this.root;
        if (entry != null) {
            while (entry.left != null) {
                entry = entry.left;
            }
            return entry;
        }
        return entry;
    }
    
    final TreeMap.Entry<K, V> getLastEntry() {
        TreeMap.Entry<Object, Object> entry = this.root;
        if (entry != null) {
            while (entry.right != null) {
                entry = entry.right;
            }
            return entry;
        }
        return entry;
    }
    
    static <K, V> TreeMap.Entry<K, V> successor(TreeMap.Entry<K, V> entry) {
        if (entry == null) {
            return null;
        }
        if (entry.right != null) {
            TreeMap.Entry<Object, Object> entry3;
            for (entry3 = entry.right; entry3.left != null; entry3 = entry3.left) {}
            return entry3;
        }
        TreeMap.Entry<Object, Object> entry2;
        for (entry2 = entry.parent, ; entry2 != null && entry == entry2.right; entry = entry2, entry2 = entry2.parent) {}
        return entry2;
    }
    
    static <K, V> TreeMap.Entry<K, V> predecessor(TreeMap.Entry<K, V> entry) {
        if (entry == null) {
            return null;
        }
        if (entry.left != null) {
            TreeMap.Entry<Object, Object> entry3;
            for (entry3 = entry.left; entry3.right != null; entry3 = entry3.right) {}
            return entry3;
        }
        TreeMap.Entry<Object, Object> entry2;
        for (entry2 = entry.parent, ; entry2 != null && entry == entry2.left; entry = entry2, entry2 = entry2.parent) {}
        return entry2;
    }
    
    private static <K, V> boolean colorOf(TreeMap.Entry<K, V> entry) {
        return entry == null || entry.color;
    }
    
    private static <K, V> TreeMap.Entry<K, V> parentOf(TreeMap.Entry<K, V> entry) {
        return (entry == null) ? null : entry.parent;
    }
    
    private static <K, V> void setColor(TreeMap.Entry<K, V> entry, boolean color) {
        if (entry != null) {
            entry.color = color;
        }
    }
    
    private static <K, V> TreeMap.Entry<K, V> leftOf(TreeMap.Entry<K, V> entry) {
        return (entry == null) ? null : entry.left;
    }
    
    private static <K, V> TreeMap.Entry<K, V> rightOf(TreeMap.Entry<K, V> entry) {
        return (entry == null) ? null : entry.right;
    }
    
    private void rotateLeft(TreeMap.Entry<K, V> entry) {
        if (entry != null) {
            TreeMap.Entry<Object, Object> right = entry.right;
            entry.right = right.left;
            if (right.left != null) {
                right.left.parent = entry;
            }
            right.parent = entry.parent;
            if (entry.parent == null) {
                this.root = right;
            }
            else if (entry.parent.left == entry) {
                entry.parent.left = right;
            }
            else {
                entry.parent.right = right;
            }
            right.left = entry;
            entry.parent = right;
        }
    }
    
    private void rotateRight(TreeMap.Entry<K, V> entry) {
        if (entry != null) {
            TreeMap.Entry<Object, Object> left = entry.left;
            entry.left = left.right;
            if (left.right != null) {
                left.right.parent = entry;
            }
            left.parent = entry.parent;
            if (entry.parent == null) {
                this.root = left;
            }
            else if (entry.parent.right == entry) {
                entry.parent.right = left;
            }
            else {
                entry.parent.left = left;
            }
            left.right = entry;
            entry.parent = left;
        }
    }
    
    private void fixAfterInsertion(TreeMap.Entry<K, V> entry) {
        entry.color = false;
        while (entry != null && entry != this.root && !entry.parent.color) {
            if (parentOf(entry) == leftOf(parentOf(parentOf(entry)))) {
                TreeMap.Entry<K, V> right = rightOf(parentOf(parentOf(entry)));
                if (!colorOf(right)) {
                    setColor(parentOf(entry), true);
                    setColor(right, true);
                    setColor(parentOf(parentOf(entry)), false);
                    entry = parentOf(parentOf(entry));
                }
                else {
                    if (entry == rightOf(parentOf(entry))) {
                        entry = parentOf(entry);
                        this.rotateLeft(entry);
                    }
                    setColor(parentOf(entry), true);
                    setColor(parentOf(parentOf(entry)), false);
                    this.rotateRight(parentOf(parentOf(entry)));
                }
            }
            else {
                TreeMap.Entry<K, V> left = leftOf(parentOf(parentOf(entry)));
                if (!colorOf(left)) {
                    setColor(parentOf(entry), true);
                    setColor(left, true);
                    setColor(parentOf(parentOf(entry)), false);
                    entry = parentOf(parentOf(entry));
                }
                else {
                    if (entry == leftOf(parentOf(entry))) {
                        entry = parentOf(entry);
                        this.rotateRight(entry);
                    }
                    setColor(parentOf(entry), true);
                    setColor(parentOf(parentOf(entry)), false);
                    this.rotateLeft(parentOf(parentOf(entry)));
                }
            }
        }
        this.root.color = true;
    }
    
    private void deleteEntry(TreeMap.Entry<K, V> entry) {
        ++this.modCount;
        --this.size;
        if (entry.left != null && entry.right != null) {
            TreeMap.Entry<K, V> successor = successor(entry);
            entry.key = successor.key;
            entry.value = successor.value;
            entry = successor;
        }
        TreeMap.Entry<K, V> root = (entry.left != null) ? entry.left : entry.right;
        if (root != null) {
            root.parent = entry.parent;
            if (entry.parent == null) {
                this.root = root;
            }
            else if (entry == entry.parent.left) {
                entry.parent.left = root;
            }
            else {
                entry.parent.right = root;
            }
            TreeMap.Entry<K, V> entry2 = entry;
            TreeMap.Entry<K, V> left = null;
            entry.parent = left;
            entry.right = left;
            entry2.left = left;
            if (entry.color) {
                this.fixAfterDeletion(root);
            }
        }
        else if (entry.parent == null) {
            this.root = null;
        }
        else {
            if (entry.color) {
                this.fixAfterDeletion(entry);
            }
            if (entry.parent != null) {
                if (entry == entry.parent.left) {
                    entry.parent.left = null;
                }
                else if (entry == entry.parent.right) {
                    entry.parent.right = null;
                }
                entry.parent = null;
            }
        }
    }
    
    private void fixAfterDeletion(TreeMap.Entry<K, V> entry) {
        while (entry != this.root && colorOf(entry)) {
            if (entry == leftOf(parentOf(entry))) {
                TreeMap.Entry<Object, Object> entry3 = rightOf(parentOf(entry));
                if (!colorOf(entry3)) {
                    setColor(entry3, true);
                    setColor(parentOf(entry), false);
                    this.rotateLeft(parentOf(entry));
                    entry3 = rightOf(parentOf(entry));
                }
                if (colorOf(leftOf(entry3)) && colorOf(rightOf(entry3))) {
                    setColor(entry3, false);
                    entry = parentOf(entry);
                }
                else {
                    if (colorOf(rightOf(entry3))) {
                        setColor(leftOf(entry3), true);
                        setColor(entry3, false);
                        this.rotateRight(entry3);
                        entry3 = rightOf(parentOf(entry));
                    }
                    setColor(entry3, colorOf(parentOf(entry)));
                    setColor(parentOf(entry), true);
                    setColor(rightOf(entry3), true);
                    this.rotateLeft(parentOf(entry));
                    entry = this.root;
                }
            }
            else {
                TreeMap.Entry<Object, Object> entry2 = leftOf(parentOf(entry));
                if (!colorOf(entry2)) {
                    setColor(entry2, true);
                    setColor(parentOf(entry), false);
                    this.rotateRight(parentOf(entry));
                    entry2 = leftOf(parentOf(entry));
                }
                if (colorOf(rightOf(entry2)) && colorOf(leftOf(entry2))) {
                    setColor(entry2, false);
                    entry = parentOf(entry);
                }
                else {
                    if (colorOf(leftOf(entry2))) {
                        setColor(rightOf(entry2), true);
                        setColor(entry2, false);
                        this.rotateLeft(entry2);
                        entry2 = leftOf(parentOf(entry));
                    }
                    setColor(entry2, colorOf(parentOf(entry)));
                    setColor(parentOf(entry), true);
                    setColor(leftOf(entry2), true);
                    this.rotateRight(parentOf(entry));
                    entry = this.root;
                }
            }
        }
        setColor(entry, true);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.size);
        for (Map.Entry entry : this.entrySet()) {
            objectOutputStream.writeObject(entry.getKey());
            objectOutputStream.writeObject(entry.getValue());
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.buildFromSorted(objectInputStream.readInt(), null, objectInputStream, null);
    }
    
    void readTreeSet(int n, ObjectInputStream objectInputStream, V v) throws IOException, ClassNotFoundException {
        this.buildFromSorted(n, null, objectInputStream, v);
    }
    
    void addAllForTreeSet(SortedSet<? extends K> set, V v) {
        try {
            this.buildFromSorted(set.size(), set.iterator(), null, v);
        }
        catch (IOException ex) {}
        catch (ClassNotFoundException ex2) {}
    }
    
    private void buildFromSorted(int size, Iterator<?> iterator, ObjectInputStream objectInputStream, V v) throws IOException, ClassNotFoundException {
        this.size = size;
        this.root = this.buildFromSorted(0, 0, size - 1, computeRedLevel(size), iterator, objectInputStream, v);
    }
    
    private final TreeMap.Entry<K, V> buildFromSorted(int n, int n2, int n3, int n4, Iterator<?> iterator, ObjectInputStream objectInputStream, V v) throws IOException, ClassNotFoundException {
        if (n3 < n2) {
            return null;
        }
        int n5 = n2 + n3 >>> 1;
        TreeMap.Entry<K, V> buildFromSorted = null;
        if (n2 < n5) {
            buildFromSorted = this.buildFromSorted(n + 1, n2, n5 - 1, n4, iterator, objectInputStream, v);
        }
        Object o;
        Object value;
        if (iterator != null) {
            if (v == null) {
                Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
                o = entry.getKey();
                value = entry.getValue();
            }
            else {
                o = iterator.next();
                value = v;
            }
        }
        else {
            o = objectInputStream.readObject();
            value = ((v != null) ? v : objectInputStream.readObject());
        }
        TreeMap.Entry o2 = new TreeMap.Entry(o, value);
        if (n == n4) {
            o2.color = false;
        }
        if (buildFromSorted != null) {
            o2.left = buildFromSorted;
            buildFromSorted.parent = o2;
        }
        if (n5 < n3) {
            TreeMap.Entry<K, V> buildFromSorted2 = this.buildFromSorted(n + 1, n5 + 1, n3, n4, iterator, objectInputStream, v);
            o2.right = buildFromSorted2;
            buildFromSorted2.parent = o2;
            return o2;
        }
        return o2;
    }
    
    private static int computeRedLevel(int n) {
        int n2 = 0;
        for (int i = n - 1; i >= 0; i = i / 2 - 1) {
            ++n2;
        }
        return n2;
    }
    
    static <K> Spliterator<K> keySpliteratorFor(NavigableMap<K, ?> navigableMap) {
        if ((navigableMap instanceof TreeMap)) {
            return ((TreeMap)navigableMap).keySpliterator();
        }
        if ((navigableMap instanceof TreeMap.DescendingSubMap)) {
            TreeMap.DescendingSubMap<Object, Object> descendingSubMap = (TreeMap.DescendingSubMap)navigableMap;
            TreeMap<Object, Object> m = descendingSubMap.m;
            if (descendingSubMap == m.descendingMap) {
                return m.descendingKeySpliterator();
            }
        }
        return ((TreeMap.NavigableSubMap)navigableMap).keySpliterator();
    }
    
    final Spliterator<K> keySpliterator() {
        return new TreeMap.KeySpliterator(this, null, null, 0, -1, 0);
    }
    
    final Spliterator<K> descendingKeySpliterator() {
        return new TreeMap.DescendingKeySpliterator(this, null, null, 0, -2, 0);
    }
    
    static void access$000(TreeMap treeMap, TreeMap.Entry entry) {
        treeMap.deleteEntry(entry);
    }
    
    static {
        UNBOUNDED = new Object();
    }
    
    class Values extends AbstractCollection<V>
    {
        @Override
        public Iterator<V> iterator() {
            return new TreeMap.ValueIterator(TreeMap.this.getFirstEntry());
        }
        
        @Override
        public int size() {
            return TreeMap.this.size();
        }
        
        @Override
        public boolean contains(Object o) {
            return TreeMap.this.containsValue(o);
        }
        
        @Override
        public boolean remove(Object o) {
            TreeMap.Entry<Object, Object> entry = TreeMap.this.getFirstEntry();
            while (entry != null) {
                if (TreeMap.valEquals(entry.getValue(), o)) {
                    TreeMap.access$000(TreeMap.this, entry);
                    return true;
                }
                entry = TreeMap.successor(entry);
            }
            return false;
        }
        
        @Override
        public void clear() {
            TreeMap.this.clear();
        }
        
        @Override
        public Spliterator<V> spliterator() {
            return new TreeMap.ValueSpliterator(TreeMap.this, null, null, 0, -1, 0);
        }
    }
    
    class EntrySet extends AbstractSet<Map.Entry<K, V>>
    {
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            return new TreeMap.EntryIterator(TreeMap.this.getFirstEntry());
        }
        
        @Override
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            Object value = entry.getValue();
            TreeMap.Entry<Object, Object> entry2 = TreeMap.this.getEntry(entry.getKey());
            return entry2 != null && TreeMap.valEquals(entry2.getValue(), value);
        }
        
        @Override
        public boolean remove(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            Object value = entry.getValue();
            TreeMap.Entry<Object, Object> entry2 = TreeMap.this.getEntry(entry.getKey());
            if (entry2 != null && TreeMap.valEquals(entry2.getValue(), value)) {
                TreeMap.access$000(TreeMap.this, entry2);
                return true;
            }
            return false;
        }
        
        @Override
        public int size() {
            return TreeMap.this.size();
        }
        
        @Override
        public void clear() {
            TreeMap.this.clear();
        }
        
        @Override
        public Spliterator<Map.Entry<K, V>> spliterator() {
            return new TreeMap.EntrySpliterator(TreeMap.this, null, null, 0, -1, 0);
        }
    }
    
    static final class KeySet<E> extends AbstractSet<E> implements NavigableSet<E>
    {
        private final NavigableMap<E, ?> m;
        
        KeySet(NavigableMap<E, ?> m) {
            this.m = m;
        }
        
        @Override
        public Iterator<E> iterator() {
            if ((this.m instanceof TreeMap)) {
                return ((TreeMap)this.m).keyIterator();
            }
            return ((TreeMap.NavigableSubMap)this.m).keyIterator();
        }
        
        @Override
        public Iterator<E> descendingIterator() {
            if ((this.m instanceof TreeMap)) {
                return ((TreeMap)this.m).descendingKeyIterator();
            }
            return ((TreeMap.NavigableSubMap)this.m).descendingKeyIterator();
        }
        
        @Override
        public int size() {
            return this.m.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.m.isEmpty();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.m.containsKey(o);
        }
        
        @Override
        public void clear() {
            this.m.clear();
        }
        
        @Override
        public E lower(E e) {
            return this.m.lowerKey(e);
        }
        
        @Override
        public E floor(E e) {
            return this.m.floorKey(e);
        }
        
        @Override
        public E ceiling(E e) {
            return this.m.ceilingKey(e);
        }
        
        @Override
        public E higher(E e) {
            return this.m.higherKey(e);
        }
        
        @Override
        public E first() {
            return this.m.firstKey();
        }
        
        @Override
        public E last() {
            return this.m.lastKey();
        }
        
        @Override
        public Comparator<? super E> comparator() {
            return this.m.comparator();
        }
        
        @Override
        public E pollFirst() {
            Map.Entry<E, Object> pollFirstEntry = this.m.pollFirstEntry();
            return (pollFirstEntry == null) ? null : pollFirstEntry.getKey();
        }
        
        @Override
        public E pollLast() {
            Map.Entry<E, Object> pollLastEntry = this.m.pollLastEntry();
            return (pollLastEntry == null) ? null : pollLastEntry.getKey();
        }
        
        @Override
        public boolean remove(Object o) {
            int size = this.size();
            this.m.remove(o);
            return this.size() != size;
        }
        
        @Override
        public NavigableSet<E> subSet(E e, boolean b, E e2, boolean b2) {
            return new TreeMap.KeySet(this.m.subMap(e, b, e2, b2));
        }
        
        @Override
        public NavigableSet<E> headSet(E e, boolean b) {
            return new TreeMap.KeySet(this.m.headMap(e, b));
        }
        
        @Override
        public NavigableSet<E> tailSet(E e, boolean b) {
            return new TreeMap.KeySet(this.m.tailMap(e, b));
        }
        
        @Override
        public SortedSet<E> subSet(E e, E e2) {
            return this.subSet(e, true, e2, false);
        }
        
        @Override
        public SortedSet<E> headSet(E e) {
            return this.headSet(e, false);
        }
        
        @Override
        public SortedSet<E> tailSet(E e) {
            return this.tailSet(e, true);
        }
        
        @Override
        public NavigableSet<E> descendingSet() {
            return new TreeMap.KeySet(this.m.descendingMap());
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return TreeMap.keySpliteratorFor(this.m);
        }
    }
    
    abstract class PrivateEntryIterator<T> implements Iterator<T>
    {
        TreeMap.Entry<K, V> next;
        TreeMap.Entry<K, V> lastReturned;
        int expectedModCount;
        
        PrivateEntryIterator(TreeMap.Entry<K, V> next) {
            this.expectedModCount = TreeMap.this.modCount;
            this.lastReturned = null;
            this.next = next;
        }
        
        @Override
        public final boolean hasNext() {
            return this.next != null;
        }
        
        final TreeMap.Entry<K, V> nextEntry() {
            TreeMap.Entry<K, V> next = this.next;
            if (next == null) {
                throw new NoSuchElementException();
            }
            if (TreeMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            this.next = TreeMap.successor(next);
            return this.lastReturned = next;
        }
        
        final TreeMap.Entry<K, V> prevEntry() {
            TreeMap.Entry<K, V> next = this.next;
            if (next == null) {
                throw new NoSuchElementException();
            }
            if (TreeMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            this.next = TreeMap.predecessor(next);
            return this.lastReturned = next;
        }
        
        @Override
        public void remove() {
            if (this.lastReturned == null) {
                throw new IllegalStateException();
            }
            if (TreeMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            if (this.lastReturned.left != null && this.lastReturned.right != null) {
                this.next = this.lastReturned;
            }
            TreeMap.access$000(TreeMap.this, this.lastReturned);
            this.expectedModCount = TreeMap.this.modCount;
            this.lastReturned = null;
            return;
        }
    }
    
    final class EntryIterator extends TreeMap.PrivateEntryIterator<Map.Entry<K, V>>
    {
        EntryIterator(TreeMap.Entry<K, V> entry) {
            super(entry);
        }
        
        @Override
        public Map.Entry<K, V> next() {
            return this.nextEntry();
        }
    }
    
    final class ValueIterator extends TreeMap.PrivateEntryIterator<V>
    {
        ValueIterator(TreeMap.Entry<K, V> entry) {
            super(entry);
        }
        
        @Override
        public V next() {
            return this.nextEntry().value;
        }
    }
    
    final class KeyIterator extends TreeMap.PrivateEntryIterator<K>
    {
        KeyIterator(TreeMap.Entry<K, V> entry) {
            super(entry);
        }
        
        @Override
        public K next() {
            return this.nextEntry().key;
        }
    }
    
    final class DescendingKeyIterator extends TreeMap.PrivateEntryIterator<K>
    {
        DescendingKeyIterator(TreeMap.Entry<K, V> entry) {
            super(entry);
        }
        
        @Override
        public K next() {
            return this.prevEntry().key;
        }
        
        @Override
        public void remove() {
            if (this.lastReturned == null) {
                throw new IllegalStateException();
            }
            if (TreeMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            TreeMap.access$000(TreeMap.this, this.lastReturned);
            this.lastReturned = null;
            this.expectedModCount = TreeMap.this.modCount;
            return;
        }
    }
    
    abstract static class NavigableSubMap<K, V> extends AbstractMap<K, V> implements NavigableMap<K, V>, Serializable
    {
        private static final long serialVersionUID = -2102997345730753016L;
        final TreeMap<K, V> m;
        final K lo;
        final K hi;
        final boolean fromStart;
        final boolean toEnd;
        final boolean loInclusive;
        final boolean hiInclusive;
        transient NavigableMap<K, V> descendingMapView;
        transient TreeMap.NavigableSubMap.EntrySetView entrySetView;
        transient TreeMap.KeySet<K> navigableKeySetView;
        
        NavigableSubMap(TreeMap<K, V> m, boolean fromStart, K lo, boolean loInclusive, boolean toEnd, K hi, boolean hiInclusive) {
            if (!fromStart && !toEnd) {
                if (m.compare(lo, hi) > 0) {
                    throw new IllegalArgumentException("fromKey > toKey");
                }
            }
            else {
                if (!fromStart) {
                    m.compare(lo, lo);
                }
                if (!toEnd) {
                    m.compare(hi, hi);
                }
            }
            this.m = m;
            this.fromStart = fromStart;
            this.lo = lo;
            this.loInclusive = loInclusive;
            this.toEnd = toEnd;
            this.hi = hi;
            this.hiInclusive = hiInclusive;
        }
        
        final boolean tooLow(Object o) {
            if (!this.fromStart) {
                int compare = this.m.compare(o, this.lo);
                if (compare < 0 || (compare == 0 && !this.loInclusive)) {
                    return true;
                }
            }
            return false;
        }
        
        final boolean tooHigh(Object o) {
            if (!this.toEnd) {
                int compare = this.m.compare(o, this.hi);
                if (compare > 0 || (compare == 0 && !this.hiInclusive)) {
                    return true;
                }
            }
            return false;
        }
        
        final boolean inRange(Object o) {
            return !this.tooLow(o) && !this.tooHigh(o);
        }
        
        final boolean inClosedRange(Object o) {
            return (this.fromStart || this.m.compare(o, this.lo) >= 0) && (this.toEnd || this.m.compare(this.hi, o) >= 0);
        }
        
        final boolean inRange(Object o, boolean b) {
            return b ? this.inRange(o) : this.inClosedRange(o);
        }
        
        final TreeMap.Entry<K, V> absLowest() {
            TreeMap.Entry<Object, Object> entry = this.fromStart ? this.m.getFirstEntry() : (this.loInclusive ? this.m.getCeilingEntry(this.lo) : this.m.getHigherEntry(this.lo));
            return (entry == null || this.tooHigh(entry.key)) ? null : entry;
        }
        
        final TreeMap.Entry<K, V> absHighest() {
            TreeMap.Entry<Object, Object> entry = this.toEnd ? this.m.getLastEntry() : (this.hiInclusive ? this.m.getFloorEntry(this.hi) : this.m.getLowerEntry(this.hi));
            return (entry == null || this.tooLow(entry.key)) ? null : entry;
        }
        
        final TreeMap.Entry<K, V> absCeiling(K k) {
            if (this.tooLow(k)) {
                return this.absLowest();
            }
            TreeMap.Entry<K, V> ceilingEntry = this.m.getCeilingEntry(k);
            return (ceilingEntry == null || this.tooHigh(ceilingEntry.key)) ? null : ceilingEntry;
        }
        
        final TreeMap.Entry<K, V> absHigher(K k) {
            if (this.tooLow(k)) {
                return this.absLowest();
            }
            TreeMap.Entry<K, V> higherEntry = this.m.getHigherEntry(k);
            return (higherEntry == null || this.tooHigh(higherEntry.key)) ? null : higherEntry;
        }
        
        final TreeMap.Entry<K, V> absFloor(K k) {
            if (this.tooHigh(k)) {
                return this.absHighest();
            }
            TreeMap.Entry<K, V> floorEntry = this.m.getFloorEntry(k);
            return (floorEntry == null || this.tooLow(floorEntry.key)) ? null : floorEntry;
        }
        
        final TreeMap.Entry<K, V> absLower(K k) {
            if (this.tooHigh(k)) {
                return this.absHighest();
            }
            TreeMap.Entry<K, V> lowerEntry = this.m.getLowerEntry(k);
            return (lowerEntry == null || this.tooLow(lowerEntry.key)) ? null : lowerEntry;
        }
        
        final TreeMap.Entry<K, V> absHighFence() {
            return this.toEnd ? null : (this.hiInclusive ? this.m.getHigherEntry(this.hi) : this.m.getCeilingEntry(this.hi));
        }
        
        final TreeMap.Entry<K, V> absLowFence() {
            return this.fromStart ? null : (this.loInclusive ? this.m.getLowerEntry(this.lo) : this.m.getFloorEntry(this.lo));
        }
        
        abstract TreeMap.Entry<K, V> subLowest();
        
        abstract TreeMap.Entry<K, V> subHighest();
        
        abstract TreeMap.Entry<K, V> subCeiling(K p0);
        
        abstract TreeMap.Entry<K, V> subHigher(K p0);
        
        abstract TreeMap.Entry<K, V> subFloor(K p0);
        
        abstract TreeMap.Entry<K, V> subLower(K p0);
        
        abstract Iterator<K> keyIterator();
        
        abstract Spliterator<K> keySpliterator();
        
        abstract Iterator<K> descendingKeyIterator();
        
        @Override
        public boolean isEmpty() {
            return (this.fromStart && this.toEnd) ? this.m.isEmpty() : this.entrySet().isEmpty();
        }
        
        @Override
        public int size() {
            return (this.fromStart && this.toEnd) ? this.m.size() : this.entrySet().size();
        }
        
        @Override
        public final boolean containsKey(Object o) {
            return this.inRange(o) && this.m.containsKey(o);
        }
        
        @Override
        public final V put(K k, V v) {
            if (!this.inRange(k)) {
                throw new IllegalArgumentException("key out of range");
            }
            return this.m.put(k, v);
        }
        
        @Override
        public final V get(Object o) {
            return this.inRange(o) ? this.m.get(o) : null;
        }
        
        @Override
        public final V remove(Object o) {
            return this.inRange(o) ? this.m.remove(o) : null;
        }
        
        @Override
        public final Map.Entry<K, V> ceilingEntry(K k) {
            return TreeMap.exportEntry(this.subCeiling(k));
        }
        
        @Override
        public final K ceilingKey(K k) {
            return TreeMap.keyOrNull(this.subCeiling(k));
        }
        
        @Override
        public final Map.Entry<K, V> higherEntry(K k) {
            return TreeMap.exportEntry(this.subHigher(k));
        }
        
        @Override
        public final K higherKey(K k) {
            return TreeMap.keyOrNull(this.subHigher(k));
        }
        
        @Override
        public final Map.Entry<K, V> floorEntry(K k) {
            return TreeMap.exportEntry(this.subFloor(k));
        }
        
        @Override
        public final K floorKey(K k) {
            return TreeMap.keyOrNull(this.subFloor(k));
        }
        
        @Override
        public final Map.Entry<K, V> lowerEntry(K k) {
            return TreeMap.exportEntry(this.subLower(k));
        }
        
        @Override
        public final K lowerKey(K k) {
            return TreeMap.keyOrNull(this.subLower(k));
        }
        
        @Override
        public final K firstKey() {
            return TreeMap.key(this.subLowest());
        }
        
        @Override
        public final K lastKey() {
            return TreeMap.key(this.subHighest());
        }
        
        @Override
        public final Map.Entry<K, V> firstEntry() {
            return TreeMap.exportEntry(this.subLowest());
        }
        
        @Override
        public final Map.Entry<K, V> lastEntry() {
            return TreeMap.exportEntry(this.subHighest());
        }
        
        @Override
        public final Map.Entry<K, V> pollFirstEntry() {
            TreeMap.Entry<K, V> subLowest = this.subLowest();
            Map.Entry<K, V> exportEntry = TreeMap.exportEntry(subLowest);
            if (subLowest != null) {
                TreeMap.access$000(this.m, subLowest);
                return exportEntry;
            }
            return exportEntry;
        }
        
        @Override
        public final Map.Entry<K, V> pollLastEntry() {
            TreeMap.Entry<K, V> subHighest = this.subHighest();
            Map.Entry<K, V> exportEntry = TreeMap.exportEntry(subHighest);
            if (subHighest != null) {
                TreeMap.access$000(this.m, subHighest);
                return exportEntry;
            }
            return exportEntry;
        }
        
        @Override
        public final NavigableSet<K> navigableKeySet() {
            TreeMap.KeySet<Object> navigableKeySetView = this.navigableKeySetView;
            return (navigableKeySetView != null) ? navigableKeySetView : (this.navigableKeySetView = new TreeMap.KeySet(this));
        }
        
        @Override
        public final Set<K> keySet() {
            return this.navigableKeySet();
        }
        
        @Override
        public NavigableSet<K> descendingKeySet() {
            return this.descendingMap().navigableKeySet();
        }
        
        @Override
        public final SortedMap<K, V> subMap(K k, K i) {
            return this.subMap(k, true, i, false);
        }
        
        @Override
        public final SortedMap<K, V> headMap(K k) {
            return this.headMap(k, false);
        }
        
        @Override
        public final SortedMap<K, V> tailMap(K k) {
            return this.tailMap(k, true);
        }
        
        abstract class EntrySetView extends AbstractSet<Map.Entry<K, V>>
        {
            private transient int size;
            private transient int sizeModCount;
            
            EntrySetView() {
                this.size = -1;
            }
            
            @Override
            public int size() {
                if (TreeMap.NavigableSubMap.this.fromStart && TreeMap.NavigableSubMap.this.toEnd) {
                    return TreeMap.NavigableSubMap.this.m.size();
                }
                if (this.size == -1 || this.sizeModCount != TreeMap.NavigableSubMap.this.m.modCount) {
                    this.sizeModCount = TreeMap.NavigableSubMap.this.m.modCount;
                    this.size = 0;
                    Iterator<Map.Entry<K, V>> iterator = this.iterator();
                    while (iterator.hasNext()) {
                        ++this.size;
                        iterator.next();
                    }
                }
                return this.size;
            }
            
            @Override
            public boolean isEmpty() {
                TreeMap.Entry<Object, Object> absLowest = TreeMap.NavigableSubMap.this.absLowest();
                return absLowest == null || TreeMap.NavigableSubMap.this.tooHigh(absLowest.key);
            }
            
            @Override
            public boolean contains(Object o) {
                if (!(o instanceof Map.Entry)) {
                    return false;
                }
                Map.Entry<Object, Object> entry = (Map.Entry)o;
                Object key = entry.getKey();
                if (!TreeMap.NavigableSubMap.this.inRange(key)) {
                    return false;
                }
                TreeMap.Entry<K, V> entry2 = TreeMap.NavigableSubMap.this.m.getEntry(key);
                return entry2 != null && TreeMap.valEquals(entry2.getValue(), entry.getValue());
            }
            
            @Override
            public boolean remove(Object o) {
                if (!(o instanceof Map.Entry)) {
                    return false;
                }
                Map.Entry<Object, Object> entry = (Map.Entry)o;
                Object key = entry.getKey();
                if (!TreeMap.NavigableSubMap.this.inRange(key)) {
                    return false;
                }
                TreeMap.Entry<K, V> entry2 = TreeMap.NavigableSubMap.this.m.getEntry(key);
                if (entry2 != null && TreeMap.valEquals(entry2.getValue(), entry.getValue())) {
                    TreeMap.access$000(TreeMap.NavigableSubMap.this.m, entry2);
                    return true;
                }
                return false;
            }
        }
        
        abstract class SubMapIterator<T> implements Iterator<T>
        {
            TreeMap.Entry<K, V> lastReturned;
            TreeMap.Entry<K, V> next;
            final Object fenceKey;
            int expectedModCount;
            
            SubMapIterator(TreeMap.Entry<K, V> next, TreeMap.Entry<K, V> entry) {
                this.expectedModCount = TreeMap.NavigableSubMap.this.m.modCount;
                this.lastReturned = null;
                this.next = next;
                this.fenceKey = ((entry == null) ? TreeMap.UNBOUNDED : entry.key);
            }
            
            @Override
            public final boolean hasNext() {
                return this.next != null && this.next.key != this.fenceKey;
            }
            
            final TreeMap.Entry<K, V> nextEntry() {
                TreeMap.Entry<K, V> next = this.next;
                if (next == null || next.key == this.fenceKey) {
                    throw new NoSuchElementException();
                }
                if (TreeMap.NavigableSubMap.this.m.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
                this.next = TreeMap.successor(next);
                return this.lastReturned = next;
            }
            
            final TreeMap.Entry<K, V> prevEntry() {
                TreeMap.Entry<K, V> next = this.next;
                if (next == null || next.key == this.fenceKey) {
                    throw new NoSuchElementException();
                }
                if (TreeMap.NavigableSubMap.this.m.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
                this.next = TreeMap.predecessor(next);
                return this.lastReturned = next;
            }
            
            final void removeAscending() {
                if (this.lastReturned == null) {
                    throw new IllegalStateException();
                }
                if (TreeMap.NavigableSubMap.this.m.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
                if (this.lastReturned.left != null && this.lastReturned.right != null) {
                    this.next = this.lastReturned;
                }
                TreeMap.access$000(TreeMap.NavigableSubMap.this.m, this.lastReturned);
                this.lastReturned = null;
                this.expectedModCount = TreeMap.NavigableSubMap.this.m.modCount;
                return;
            }
            
            final void removeDescending() {
                if (this.lastReturned == null) {
                    throw new IllegalStateException();
                }
                if (TreeMap.NavigableSubMap.this.m.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
                TreeMap.access$000(TreeMap.NavigableSubMap.this.m, this.lastReturned);
                this.lastReturned = null;
                this.expectedModCount = TreeMap.NavigableSubMap.this.m.modCount;
                return;
            }
        }
        
        final class SubMapEntryIterator extends TreeMap.NavigableSubMap.SubMapIterator<Map.Entry<K, V>>
        {
            SubMapEntryIterator(TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2) {
                super(entry, entry2);
            }
            
            @Override
            public Map.Entry<K, V> next() {
                return this.nextEntry();
            }
            
            @Override
            public void remove() {
                this.removeAscending();
            }
        }
        
        final class DescendingSubMapEntryIterator extends TreeMap.NavigableSubMap.SubMapIterator<Map.Entry<K, V>>
        {
            DescendingSubMapEntryIterator(TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2) {
                super(entry, entry2);
            }
            
            @Override
            public Map.Entry<K, V> next() {
                return this.prevEntry();
            }
            
            @Override
            public void remove() {
                this.removeDescending();
            }
        }
        
        final class SubMapKeyIterator extends TreeMap.NavigableSubMap.SubMapIterator<K> implements Spliterator<K>
        {
            SubMapKeyIterator(TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2) {
                super(entry, entry2);
            }
            
            @Override
            public K next() {
                return this.nextEntry().key;
            }
            
            @Override
            public void remove() {
                this.removeAscending();
            }
            
            @Override
            public Spliterator<K> trySplit() {
                return null;
            }
            
            @Override
            public void forEachRemaining(Consumer<? super K> consumer) {
                while (this.hasNext()) {
                    consumer.accept(this.next());
                }
            }
            
            @Override
            public boolean tryAdvance(Consumer<? super K> consumer) {
                if (this.hasNext()) {
                    consumer.accept(this.next());
                    return true;
                }
                return false;
            }
            
            @Override
            public long estimateSize() {
                return Long.MAX_VALUE;
            }
            
            @Override
            public int characteristics() {
                return 21;
            }
            
            @Override
            public final Comparator<? super K> getComparator() {
                return TreeMap.NavigableSubMap.this.comparator();
            }
        }
        
        final class DescendingSubMapKeyIterator extends TreeMap.NavigableSubMap.SubMapIterator<K> implements Spliterator<K>
        {
            DescendingSubMapKeyIterator(TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2) {
                super(entry, entry2);
            }
            
            @Override
            public K next() {
                return this.prevEntry().key;
            }
            
            @Override
            public void remove() {
                this.removeDescending();
            }
            
            @Override
            public Spliterator<K> trySplit() {
                return null;
            }
            
            @Override
            public void forEachRemaining(Consumer<? super K> consumer) {
                while (this.hasNext()) {
                    consumer.accept(this.next());
                }
            }
            
            @Override
            public boolean tryAdvance(Consumer<? super K> consumer) {
                if (this.hasNext()) {
                    consumer.accept(this.next());
                    return true;
                }
                return false;
            }
            
            @Override
            public long estimateSize() {
                return Long.MAX_VALUE;
            }
            
            @Override
            public int characteristics() {
                return 17;
            }
        }
    }
    
    static final class AscendingSubMap<K, V> extends TreeMap.NavigableSubMap<K, V>
    {
        private static final long serialVersionUID = 912986545866124060L;
        
        AscendingSubMap(TreeMap<K, V> treeMap, boolean b, K k, boolean b2, boolean b3, K i, boolean b4) {
            super(treeMap, b, k, b2, b3, i, b4);
        }
        
        @Override
        public Comparator<? super K> comparator() {
            return this.m.comparator();
        }
        
        @Override
        public NavigableMap<K, V> subMap(K k, boolean b, K i, boolean b2) {
            if (!this.inRange(k, b)) {
                throw new IllegalArgumentException("fromKey out of range");
            }
            if (!this.inRange(i, b2)) {
                throw new IllegalArgumentException("toKey out of range");
            }
            return new TreeMap.AscendingSubMap(this.m, false, k, b, false, i, b2);
        }
        
        @Override
        public NavigableMap<K, V> headMap(K k, boolean b) {
            if (!this.inRange(k, b)) {
                throw new IllegalArgumentException("toKey out of range");
            }
            return new TreeMap.AscendingSubMap(this.m, this.fromStart, this.lo, this.loInclusive, false, k, b);
        }
        
        @Override
        public NavigableMap<K, V> tailMap(K k, boolean b) {
            if (!this.inRange(k, b)) {
                throw new IllegalArgumentException("fromKey out of range");
            }
            return new TreeMap.AscendingSubMap(this.m, false, k, b, this.toEnd, this.hi, this.hiInclusive);
        }
        
        @Override
        public NavigableMap<K, V> descendingMap() {
            NavigableMap<Object, Object> descendingMapView = this.descendingMapView;
            return (descendingMapView != null) ? descendingMapView : (this.descendingMapView = new TreeMap.DescendingSubMap(this.m, this.fromStart, this.lo, this.loInclusive, this.toEnd, this.hi, this.hiInclusive));
        }
        
        @Override
        Iterator<K> keyIterator() {
            return new TreeMap.NavigableSubMap.SubMapKeyIterator(this.absLowest(), this.absHighFence());
        }
        
        @Override
        Spliterator<K> keySpliterator() {
            return new TreeMap.NavigableSubMap.SubMapKeyIterator(this.absLowest(), this.absHighFence());
        }
        
        @Override
        Iterator<K> descendingKeyIterator() {
            return new TreeMap.NavigableSubMap.DescendingSubMapKeyIterator(this.absHighest(), this.absLowFence());
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            TreeMap.NavigableSubMap.EntrySetView entrySetView = this.entrySetView;
            return (entrySetView != null) ? entrySetView : (this.entrySetView = new TreeMap.AscendingSubMap.AscendingEntrySetView());
        }
        
        @Override
        TreeMap.Entry<K, V> subLowest() {
            return this.absLowest();
        }
        
        @Override
        TreeMap.Entry<K, V> subHighest() {
            return this.absHighest();
        }
        
        @Override
        TreeMap.Entry<K, V> subCeiling(K k) {
            return this.absCeiling(k);
        }
        
        @Override
        TreeMap.Entry<K, V> subHigher(K k) {
            return this.absHigher(k);
        }
        
        @Override
        TreeMap.Entry<K, V> subFloor(K k) {
            return this.absFloor(k);
        }
        
        @Override
        TreeMap.Entry<K, V> subLower(K k) {
            return this.absLower(k);
        }
        
        final class AscendingEntrySetView extends TreeMap.NavigableSubMap.EntrySetView
        {
            @Override
            public Iterator<Map.Entry<K, V>> iterator() {
                return new TreeMap.NavigableSubMap.SubMapEntryIterator(TreeMap.AscendingSubMap.this.absLowest(), TreeMap.AscendingSubMap.this.absHighFence());
            }
        }
    }
    
    static final class DescendingSubMap<K, V> extends TreeMap.NavigableSubMap<K, V>
    {
        private static final long serialVersionUID = 912986545866120460L;
        private final Comparator<? super K> reverseComparator;
        
        DescendingSubMap(TreeMap<K, V> treeMap, boolean b, K k, boolean b2, boolean b3, K i, boolean b4) {
            super(treeMap, b, k, b2, b3, i, b4);
            this.reverseComparator = Collections.reverseOrder(this.m.comparator);
        }
        
        @Override
        public Comparator<? super K> comparator() {
            return this.reverseComparator;
        }
        
        @Override
        public NavigableMap<K, V> subMap(K k, boolean b, K i, boolean b2) {
            if (!this.inRange(k, b)) {
                throw new IllegalArgumentException("fromKey out of range");
            }
            if (!this.inRange(i, b2)) {
                throw new IllegalArgumentException("toKey out of range");
            }
            return new TreeMap.DescendingSubMap(this.m, false, i, b2, false, k, b);
        }
        
        @Override
        public NavigableMap<K, V> headMap(K k, boolean b) {
            if (!this.inRange(k, b)) {
                throw new IllegalArgumentException("toKey out of range");
            }
            return new TreeMap.DescendingSubMap(this.m, false, k, b, this.toEnd, this.hi, this.hiInclusive);
        }
        
        @Override
        public NavigableMap<K, V> tailMap(K k, boolean b) {
            if (!this.inRange(k, b)) {
                throw new IllegalArgumentException("fromKey out of range");
            }
            return new TreeMap.DescendingSubMap(this.m, this.fromStart, this.lo, this.loInclusive, false, k, b);
        }
        
        @Override
        public NavigableMap<K, V> descendingMap() {
            NavigableMap<Object, Object> descendingMapView = this.descendingMapView;
            return (descendingMapView != null) ? descendingMapView : (this.descendingMapView = new TreeMap.AscendingSubMap(this.m, this.fromStart, this.lo, this.loInclusive, this.toEnd, this.hi, this.hiInclusive));
        }
        
        @Override
        Iterator<K> keyIterator() {
            return new TreeMap.NavigableSubMap.DescendingSubMapKeyIterator(this.absHighest(), this.absLowFence());
        }
        
        @Override
        Spliterator<K> keySpliterator() {
            return new TreeMap.NavigableSubMap.DescendingSubMapKeyIterator(this.absHighest(), this.absLowFence());
        }
        
        @Override
        Iterator<K> descendingKeyIterator() {
            return new TreeMap.NavigableSubMap.SubMapKeyIterator(this.absLowest(), this.absHighFence());
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            TreeMap.NavigableSubMap.EntrySetView entrySetView = this.entrySetView;
            return (entrySetView != null) ? entrySetView : (this.entrySetView = new TreeMap.DescendingSubMap.DescendingEntrySetView());
        }
        
        @Override
        TreeMap.Entry<K, V> subLowest() {
            return this.absHighest();
        }
        
        @Override
        TreeMap.Entry<K, V> subHighest() {
            return this.absLowest();
        }
        
        @Override
        TreeMap.Entry<K, V> subCeiling(K k) {
            return this.absFloor(k);
        }
        
        @Override
        TreeMap.Entry<K, V> subHigher(K k) {
            return this.absLower(k);
        }
        
        @Override
        TreeMap.Entry<K, V> subFloor(K k) {
            return this.absCeiling(k);
        }
        
        @Override
        TreeMap.Entry<K, V> subLower(K k) {
            return this.absHigher(k);
        }
        
        final class DescendingEntrySetView extends TreeMap.NavigableSubMap.EntrySetView
        {
            @Override
            public Iterator<Map.Entry<K, V>> iterator() {
                return new TreeMap.NavigableSubMap.DescendingSubMapEntryIterator(TreeMap.DescendingSubMap.this.absHighest(), TreeMap.DescendingSubMap.this.absLowFence());
            }
        }
    }
    
    private class SubMap extends AbstractMap<K, V> implements SortedMap<K, V>, Serializable
    {
        private static final long serialVersionUID = -6520786458950516097L;
        private boolean fromStart;
        private boolean toEnd;
        private K fromKey;
        private K toKey;
        
        private SubMap() {
            this.fromStart = false;
            this.toEnd = false;
        }
        
        private Object readResolve() {
            return new TreeMap.AscendingSubMap(TreeMap.this, this.fromStart, this.fromKey, true, this.toEnd, this.toKey, false);
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            throw new InternalError();
        }
        
        @Override
        public K lastKey() {
            throw new InternalError();
        }
        
        @Override
        public K firstKey() {
            throw new InternalError();
        }
        
        @Override
        public SortedMap<K, V> subMap(K k, K i) {
            throw new InternalError();
        }
        
        @Override
        public SortedMap<K, V> headMap(K k) {
            throw new InternalError();
        }
        
        @Override
        public SortedMap<K, V> tailMap(K k) {
            throw new InternalError();
        }
        
        @Override
        public Comparator<? super K> comparator() {
            throw new InternalError();
        }
    }
    
    static final class Entry<K, V> implements Map.Entry<K, V>
    {
        K key;
        V value;
        TreeMap.Entry<K, V> left;
        TreeMap.Entry<K, V> right;
        TreeMap.Entry<K, V> parent;
        boolean color;
        
        Entry(K key, V value, TreeMap.Entry<K, V> parent) {
            this.color = true;
            this.key = key;
            this.value = value;
            this.parent = parent;
        }
        
        @Override
        public K getKey() {
            return this.key;
        }
        
        @Override
        public V getValue() {
            return this.value;
        }
        
        @Override
        public V setValue(V value) {
            Object value2 = this.value;
            this.value = value;
            return value2;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return TreeMap.valEquals(this.key, entry.getKey()) && TreeMap.valEquals(this.value, entry.getValue());
        }
        
        @Override
        public int hashCode() {
            return ((this.key == null) ? 0 : this.key.hashCode()) ^ ((this.value == null) ? 0 : this.value.hashCode());
        }
        
        @Override
        public String toString() {
            return this.key + "=" + this.value;
        }
    }
    
    static class TreeMapSpliterator<K, V>
    {
        final TreeMap<K, V> tree;
        TreeMap.Entry<K, V> current;
        TreeMap.Entry<K, V> fence;
        int side;
        int est;
        int expectedModCount;
        
        TreeMapSpliterator(TreeMap<K, V> tree, TreeMap.Entry<K, V> current, TreeMap.Entry<K, V> fence, int side, int est, int expectedModCount) {
            this.tree = tree;
            this.current = current;
            this.fence = fence;
            this.side = side;
            this.est = est;
            this.expectedModCount = expectedModCount;
        }
        
        final int getEstimate() {
            int est;
            if ((est = this.est) < 0) {
                TreeMap<Object, Object> tree;
                if ((tree = this.tree) != null) {
                    this.current = ((est == -1) ? tree.getFirstEntry() : tree.getLastEntry());
                    int access$400 = tree.size;
                    this.est = access$400;
                    est = access$400;
                    this.expectedModCount = tree.modCount;
                }
                else {
                    int est2 = 0;
                    this.est = est2;
                    est = est2;
                }
            }
            return est;
        }
        
        public final long estimateSize() {
            return (long)this.getEstimate();
        }
    }
    
    static final class KeySpliterator<K, V> extends TreeMap.TreeMapSpliterator<K, V> implements Spliterator<K>
    {
        KeySpliterator(TreeMap<K, V> treeMap, TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2, int n, int n2, int n3) {
            super(treeMap, entry, entry2, n, n2, n3);
        }
        
        @Override
        public TreeMap.KeySpliterator<K, V> trySplit() {
            if (this.est < 0) {
                this.getEstimate();
            }
            int side = this.side;
            TreeMap.Entry<Object, Object> current = this.current;
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> current2 = (current == null || current == fence) ? null : ((side == 0) ? this.tree.root : ((side > 0) ? current.right : ((side < 0 && fence != null) ? fence.left : null)));
            if (current2 != null && current2 != current && current2 != fence && this.tree.compare(current.key, current2.key) < 0) {
                this.side = 1;
                TreeMap<K, V> tree = this.tree;
                TreeMap.Entry<K, V> entry = current;
                this.current = current2;
                return new TreeMap.KeySpliterator<K, V>(tree, entry, current2, -1, this.est >>>= 1, this.expectedModCount);
            }
            return null;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> entry;
            if (this.current != null && entry != fence) {
                this.current = fence;
                do {
                    consumer.accept(entry.key);
                    if ((entry = entry.right) != null) {
                        TreeMap.Entry<Object, Object> left;
                        while ((left = entry.left) != null) {
                            entry = left;
                        }
                    }
                    else {
                        while ((entry = entry.parent) != null && entry == entry.right) {}
                    }
                } while (entry != null && entry != fence);
                if (this.tree.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<K, V> current;
            if ((current = this.current) == null || current == this.fence) {
                return false;
            }
            this.current = TreeMap.successor(current);
            consumer.accept(current.key);
            if (this.tree.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return true;
        }
        
        @Override
        public int characteristics() {
            return ((this.side == 0) ? 64 : 0) | 0x1 | 0x4 | 0x10;
        }
        
        @Override
        public final Comparator<? super K> getComparator() {
            return this.tree.comparator;
        }
    }
    
    static final class DescendingKeySpliterator<K, V> extends TreeMap.TreeMapSpliterator<K, V> implements Spliterator<K>
    {
        DescendingKeySpliterator(TreeMap<K, V> treeMap, TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2, int n, int n2, int n3) {
            super(treeMap, entry, entry2, n, n2, n3);
        }
        
        @Override
        public TreeMap.DescendingKeySpliterator<K, V> trySplit() {
            if (this.est < 0) {
                this.getEstimate();
            }
            int side = this.side;
            TreeMap.Entry<Object, Object> current = this.current;
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> current2 = (current == null || current == fence) ? null : ((side == 0) ? this.tree.root : ((side < 0) ? current.left : ((side > 0 && fence != null) ? fence.right : null)));
            if (current2 != null && current2 != current && current2 != fence && this.tree.compare(current.key, current2.key) > 0) {
                this.side = 1;
                TreeMap<K, V> tree = this.tree;
                TreeMap.Entry<K, V> entry = current;
                this.current = current2;
                return new TreeMap.DescendingKeySpliterator<K, V>(tree, entry, current2, -1, this.est >>>= 1, this.expectedModCount);
            }
            return null;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> entry;
            if (this.current != null && entry != fence) {
                this.current = fence;
                do {
                    consumer.accept(entry.key);
                    if ((entry = entry.left) != null) {
                        TreeMap.Entry<Object, Object> right;
                        while ((right = entry.right) != null) {
                            entry = right;
                        }
                    }
                    else {
                        while ((entry = entry.parent) != null && entry == entry.left) {}
                    }
                } while (entry != null && entry != fence);
                if (this.tree.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<K, V> current;
            if ((current = this.current) == null || current == this.fence) {
                return false;
            }
            this.current = TreeMap.predecessor(current);
            consumer.accept(current.key);
            if (this.tree.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return true;
        }
        
        @Override
        public int characteristics() {
            return ((this.side == 0) ? 64 : 0) | 0x1 | 0x10;
        }
    }
    
    static final class ValueSpliterator<K, V> extends TreeMap.TreeMapSpliterator<K, V> implements Spliterator<V>
    {
        ValueSpliterator(TreeMap<K, V> treeMap, TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2, int n, int n2, int n3) {
            super(treeMap, entry, entry2, n, n2, n3);
        }
        
        @Override
        public TreeMap.ValueSpliterator<K, V> trySplit() {
            if (this.est < 0) {
                this.getEstimate();
            }
            int side = this.side;
            TreeMap.Entry<Object, Object> current = this.current;
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> current2 = (current == null || current == fence) ? null : ((side == 0) ? this.tree.root : ((side > 0) ? current.right : ((side < 0 && fence != null) ? fence.left : null)));
            if (current2 != null && current2 != current && current2 != fence && this.tree.compare(current.key, current2.key) < 0) {
                this.side = 1;
                TreeMap<K, V> tree = this.tree;
                TreeMap.Entry<K, V> entry = current;
                this.current = current2;
                return new TreeMap.ValueSpliterator<K, V>(tree, entry, current2, -1, this.est >>>= 1, this.expectedModCount);
            }
            return null;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> entry;
            if (this.current != null && entry != fence) {
                this.current = fence;
                do {
                    consumer.accept(entry.value);
                    if ((entry = entry.right) != null) {
                        TreeMap.Entry<Object, Object> left;
                        while ((left = entry.left) != null) {
                            entry = left;
                        }
                    }
                    else {
                        while ((entry = entry.parent) != null && entry == entry.right) {}
                    }
                } while (entry != null && entry != fence);
                if (this.tree.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<K, V> current;
            if ((current = this.current) == null || current == this.fence) {
                return false;
            }
            this.current = TreeMap.successor(current);
            consumer.accept(current.value);
            if (this.tree.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return true;
        }
        
        @Override
        public int characteristics() {
            return ((this.side == 0) ? 64 : 0) | 0x10;
        }
    }
    
    static final class EntrySpliterator<K, V> extends TreeMap.TreeMapSpliterator<K, V> implements Spliterator<Map.Entry<K, V>>
    {
        EntrySpliterator(TreeMap<K, V> treeMap, TreeMap.Entry<K, V> entry, TreeMap.Entry<K, V> entry2, int n, int n2, int n3) {
            super(treeMap, entry, entry2, n, n2, n3);
        }
        
        @Override
        public TreeMap.EntrySpliterator<K, V> trySplit() {
            if (this.est < 0) {
                this.getEstimate();
            }
            int side = this.side;
            TreeMap.Entry<Object, Object> current = this.current;
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> current2 = (current == null || current == fence) ? null : ((side == 0) ? this.tree.root : ((side > 0) ? current.right : ((side < 0 && fence != null) ? fence.left : null)));
            if (current2 != null && current2 != current && current2 != fence && this.tree.compare(current.key, current2.key) < 0) {
                this.side = 1;
                TreeMap<K, V> tree = this.tree;
                TreeMap.Entry<K, V> entry = current;
                this.current = current2;
                return new TreeMap.EntrySpliterator<K, V>(tree, entry, current2, -1, this.est >>>= 1, this.expectedModCount);
            }
            return null;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<Object, Object> fence = this.fence;
            TreeMap.Entry<Object, Object> entry;
            if (this.current != null && entry != fence) {
                this.current = fence;
                do {
                    consumer.accept(entry);
                    if ((entry = entry.right) != null) {
                        TreeMap.Entry<Object, Object> left;
                        while ((left = entry.left) != null) {
                            entry = left;
                        }
                    }
                    else {
                        while ((entry = entry.parent) != null && entry == entry.right) {}
                    }
                } while (entry != null && entry != fence);
                if (this.tree.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            if (this.est < 0) {
                this.getEstimate();
            }
            TreeMap.Entry<K, V> current;
            if ((current = this.current) == null || current == this.fence) {
                return false;
            }
            this.current = TreeMap.successor(current);
            consumer.accept(current);
            if (this.tree.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return true;
        }
        
        @Override
        public int characteristics() {
            return ((this.side == 0) ? 64 : 0) | 0x1 | 0x4 | 0x10;
        }
        
        @Override
        public Comparator<Map.Entry<K, V>> getComparator() {
            if (this.tree.comparator != null) {
                return Map.Entry.comparingByKey(this.tree.comparator);
            }
            return (Comparator)(Serializable)((entry, entry2)/* java.util.Map.Entry,java.util.Map.Entry, */ -> ((Comparable)entry.getKey()).compareTo(entry2.getKey()));
        }
    }
}
